 function [trace,error,varargout]=loadTCVdata(shot,data_type,varargin)
%
% list of data_type currently available:
% 'Ip'   =  current
% 'zmag' =  vertical position of the center of the plasma (magnetic axis)
% 'rmag' =  radial position of the center of the plasma 
% 'ece'  =  electron cyclotron emission  
% 'sxr'  =  soft x-ray emission
% 'sxR'  =  soft x-ray emission with varargout{1} option (requires varargin{4}!)
% 'MPX'  =  soft x-ray from wire chambers
%
% 'xx_2 or xx_3' for Liuqe2 or 3: same as above for xx related to equilibrium
%
% INPUT:
% shot: shot number
% data_type: type of the required data
%
% Definition of varargin depends on data_type:
%
% data_type=sxr or ece:
%                  varargin{1}:  channel status 1= unread yet, 0= read
%                                (for traces with many channel, enables to load additional channels,
%                                 like SXR, ECE, etc.)
%                  varargin{2}:  [i1 i2] : if not empty, assumes need many chords from i1 to i2
%                  varargin{3}:  zmag for varargout{1} computation
%
% OUTPUT:
%
% trace.data:   data structure
% trace.t:      time of reference 
% trace.x:      space of reference 
%
% Additional Output arguments depending on data_type
%
% data_type=sxR, ece:
%                varargout{1}: major radius: intersection/projection of the view lines with z=zmag
% data_type=MPX: 
%                varargout{1}: te be determined
%
%
% function needed: mds functions-xtomo_geometry-get_xtomo_data (Furno's routines)
%                  VsxrTCVradius
% Example:
%         [ip,error]=loadTCVdata(shot,'Ip',1);
%         [sxr,error,R]=loadTCVdata(shot,'sxR',1);
%

varargout{1}=cell(1,1);
error=1;

% some defaults

% nodes which have _2 and _3 equivalence
liuqe23=[{'\results::i_p'} {'\results::z_axis'} {'\results::r_axis'} {'\results::q_psi'} ...
      {'\results::beta_tor'} {'\results::q_95'} {'\results::l_i'}  {'\results::delta_95'} ...
      {'\results::kappa_95'} {'\results::r_contour'} {'\results::z_contour'} {'\results::psi_axis'} ...
      {'\results::thomson:psiscatvol'} {'\results::thomson:psi_max'} {'\results::psitbx:vol'}];

% all keywords and corresponding case to run below
TCVkeywrdall=[{'Ip'} {'zmag'} {'rmag'} {'sxr'} {'sxR'} {'ece'}];
TCVsig.iip=1;
TCVsig.izmag=2;
TCVsig.irmag=3;
TCVsig.isxr=4;
TCVsig.isxR=5;
TCVsig.iece=6;
% For each keyword, specify which case to use. As most common is 'simpletdi', fill in with this and change
% only indices needed. Usually use name of case same as keyword name
TCVkeywrdcase=cell(size(TCVkeywrdall));
TCVkeywrdcase(:)={'simpletdi'};
TCVkeywrdcase(TCVsig.isxr)=TCVkeywrdall(TCVsig.isxr);
TCVkeywrdcase(TCVsig.isxR)=TCVkeywrdall(TCVsig.isxR);
TCVkeywrdcase(TCVsig.iece)=TCVkeywrdall(TCVsig.iece);

% For the 'simpletdi' cases, we need the full node in case gdat was called with full location directly
% for the other cases, leave this location empty
TCVsiglocation=cell(size(TCVkeywrdall));
TCVsiglocation(:)={''};
TCVsiglocation(TCVsig.iip)={'\results::i_p'};
TCVsiglocation(TCVsig.izmag)={'\results::z_axis'};
TCVsiglocation(TCVsig.irmag)={'\results::r_axis'};

% Information about which dimension has time, always return 2D data as (x,t) array
% as most are 1D arrays with time as first index, fill in with ones and change only those needed
TCVsigtimeindx=ones(size(TCVkeywrdall));

% find index of signal called upon
i_23=0;
if strcmp(data_type(end-1:end),'_2') | strcmp(data_type(end-1:end),'_3')
  i_23=2;
end
if strcmp(data_type(1:1),'\')
  % in case full node name was given
  index=strmatch(data_type(1:end-i_23),TCVsiglocation,'exact');
  if isempty(index)
    disp('********************')
    disp('trace not yet registered.')
    disp('If standard data, ask andrea.scarabosio@epfl.ch or olivier.sauter@epfl.ch to create a keyqord entry for this data')
    eval(['!mail -s ''' data_type ' ' num2str(shot) ' ' getenv('USER') ' TCV'' olivier.sauter@epfl.ch < /dev/null'])
    disp('********************')
    % temporarily add entry in arrays, so can work below
    index=length(TCVkeywrdall)+1;
    TCVkeywrdall(end+1)={'new'};
    TCVkeywrdcase(end+1)={'simpletdi'};
    TCVsiglocation(end+1)={data_type(1:end-i_23)};
    TCVsigtimeindx(end+1)=0;
  elseif ~strcmp(TCVkeywrdcase{index},'simpletdi')
    msgbox(['Problem in loadTCVdata with data_type = ' data_type ...
          '. Full paths of nodes should only be for case simpletdi'],'in loadTCVdata','error')
    error('in loadTCVdata')
  end
else
  index=strmatch(data_type(1:end-i_23),TCVkeywrdall,'exact');
  if isempty(index)
    disp(' ')
    disp('********************')
    disp(['no such keyword: ' data_type(1:end-i_23)])
    disp(' ')
    disp('Available keywords:')
    TCVkeywrdall(:)
    disp('********************')
    trace.data=[];
    trace.t=[];
    trace.x=[];
    return
  end
end
disp(['loading' ' ' data_type ' from TCV shot #' num2str(shot)]); 
disp(['case ' TCVkeywrdcase{index}])
if i_23==2 & isempty(strmatch(TCVsiglocation(index),liuqe23,'exact'))
  disp('********')
  disp('Warning asks for liuqe 2 or 3 of a signal, but not in liuqe23 list in loadTCVdata')
  eval(['!mail -s ''' data_type ' ' num2str(shot) ' ' getenv('USER') ' TCV: not in liuqe23 list'' olivier.sauter@epfl.ch < /dev/null'])
  disp('********')
end

status=ones(1,100);
zmag=cell(0,0);
nargineff=nargin;
i1=1;
i2=20;
if nargineff>=3
  for i=1:length(varargin)
    if ~isempty(varargin{i})
      if isstruct(varargin{i})
         zmag=varargin{i};
      elseif size(varargin{i},2)>2
         status=varargin{i};
      else
         i1 =varargin{i}(1);
    	 i2 =varargin{i}(2);
      end
    end 
  end
end

switch TCVkeywrdcase{index}

      
     
  case 'simpletdi'

    %  load TCV other data 
    mdsopen(shot);
    nodenameeff=TCVsiglocation{index};
    if i_23==2
      nodenameeff=[nodenameeff data_type(end-1:end)]
    end
    % test if node exists
    error=1;
    if eval(['~mdsdata(''node_exists("\' nodenameeff '")'')'])
      disp(['node ' nodenameeff ' does not exist for shot = ' num2str(shot)])
      trace.data=[];
      trace.t=[];
      trace.x=[];
      return
    elseif eval(['mdsdata(''getnci("\' nodenameeff ':foo","length")'')==0'])
      disp(['no data for node ' nodenameeff ' for shot = ' num2str(shot)])
      trace.data=[];
      trace.t=[];
      trace.x=[];
      return
    end
    tracetdi=tdi(nodenameeff);
    mdsclose(shot)
    if isempty(tracetdi.data) | isnan(tracetdi.data)
      disp(['node ' nodenameeff ' is empty for shot = ' num2str(shot)])
      trace.data=[];
      trace.t=[];
      trace.x=[];
      return
    end
    trace.data=tracetdi.data;
    trace.t=tracetdi.dim{max(1,TCVsigtimeindx(index))};
    if length(tracetdi.dim)==2
      % 2D data
      ix=2;
      if TCVsigtimeindx(index)==2;
        ix=1;
      elseif TCVsigtimeindx(index)==0;
        % time array unknow, assumes time array with values having most number of digits
        ab1=num2str(tracetdi.dim{1}-fix(tracetdi.dim{1}));
        ab2=num2str(tracetdi.dim{2}-fix(tracetdi.dim{2}));
        if size(ab1,2)<size(ab2,2); ix=1; end
        disp(['assumes time array has more digits, so x from dim{' num2str(ix) '}'])
      end;
      trace.x=tracetdi.dim{ix};
      % make sure data is of (x,t)
      if ix==2
        % transpose data
        trace.data=trace.data';
        if TCVsigtimeindx(index)==2
          trace.t=tracetdi.dim{1};
        end
      else
        if max(1,TCVsigtimeindx(index))==1
          trace.t=tracetdi.dim{2};
        end
      end
    elseif length(tracetdi.dim)>2
      msgbox(['data more than 2D (data_type=' data_type ...
            '), check how to save it, contact andrea.scarabosio@epfl.ch or olivier.sauter@epfl.ch'],...
          'in simpletdi','warn')
      warning('in simpletdi of loadTCVdata')
    else
      trace.x=[]; 
      if max(1,TCVsigtimeindx(index))~=1
        disp('Problems in loadTCVdata, max(1,TCVsigtimeindx(index)) should be 1')
      end
    end 
    error=0;
   
  case {'sxr','sxR'}
    %  load TCV soft x-ray data  
    if isempty(zmag)
      zmag=loadTCVdata(shot,'zmag');
    end

    % camera selection: 1-10. each camera has 20 channels
    icamera=[0 1 0 0 0 0 0 0 0 0];  %index of the camera to use
    if ~isempty(find(status(1:20*icamera*ones(10,1)) == 1))
      [fans,vangle,xchord,ychord,aomega,angfact]=xtomo_geometry(1,icamera);
      % calculating intersection of the view lines with magnetic axis 
      if strcmp(data_type,'sxR') 
        varargout{1}={VsxrTCVradius(zmag.data,xchord,ychord)};
        t_1=zmag.t(1);
        t_2=zmag.t(end);
      else
        t_1=0.001;
        t_2=3;
      end
      [xtomo_signal,t]=get_xtomo_data(shot,t_1,t_2,13e-6*16, ...
          icamera,angfact);
    end  
    for i=1:(20*icamera*ones(10,1))
      trace.t(:,i)=t';
    end  
    trace.data=xtomo_signal';          
    error=0;

  case 'ece'
    %  load TCV ECE data
    if isempty(zmag)
      zmag=loadTCVdata(shot,'zmag');
    end
    % Status=1 => Not Read Yet  
    if ~isempty(find(status == 1))
      [TE_ECE,TE_ECE_ERR,RHO,R,T,TE_THOM,TE_THOM_ERR,Fcentral,CAL]=ece_te ...
          (shot,[0.1 0.29],10,10);
    end  
    a=min(find(R(:,1)>=0));
    b=max(find(R(:,1)>=0));
    for i=1:size(TE_ECE,2)
      trace.t(:,i)=T(a:b);
    end  
    trace.data=TE_ECE(a:b,:);
    radius.t=trace.t; 
    radius.data=R(a:b,:);
    varargout{1}={radius};
    error=0;
 
  case 'MPX'
    %  load TCV MPX data
    % Status=1 => Not Read Yet 
    t_1=zmag.t(1);
    t_2=zmag.t(end);
    if ~isempty(find(status == 1))
      mdsopen(shot);
      signal=get_mds_mio('MPX',[t_1 t_2]); 
      mdsclose(shot)
      trace.data=signal.data;
      for i=1:size(signal.dim{2},2)
        trace.t(:,i)=signal.dim{1};
      end  
    end  
    [xchord,ychord]=mpx_geometry;
    varargout{1}={VsxrTCVradius(zmag.data,xchord,ychord)};
    error=0;  
    
  otherwise
    % eval(['!mailto_Andrea ''from loadTCVdata, data_type= ' data_type ''''])
    disp(['this data_type' ' ' data_type ' ' 'not yet programmed in loadTCVdata, ask Andrea.Scarabosio@epfl.ch']);

end
